package dev.peerat.mapping.providers.mysql;

import java.util.ArrayList;
import java.util.List;

import dev.peerat.parser.java.Function;
import dev.peerat.parser.java.Value;
import dev.peerat.parser.java.Value.BiValue;
import dev.peerat.parser.java.Value.LambdaValue;
import dev.peerat.parser.java.Value.MethodCallValue;

public class MySQLRequestBuilder{
	
	private Function function;
	
	private List<MethodCallValue> calls;
	private String select;
	private String where;
	
	public MySQLRequestBuilder(Function function, List<MethodCallValue> calls){
		this.function = function;
		this.calls = calls;
	}
	
	public Function getFunction(){
		return this.function;
	}
	
	private String toStringOperation(MethodCallValue value){
		String method = value.getToken().getValue();
		if(method.equals("selectAll()")){
			return "SELECT *";
		}
		if(method.equals("select()")){
			LambdaValue include = (LambdaValue) value.getParameters().get(0);
			MethodCallValue keep = (MethodCallValue)include.getOperations().get(0);
			List<String> binds = new ArrayList<>();
			for(Value param : keep.getParameters()){
				binds.add(param.getToken().getValue());
			}
			String result = ""; //optimise with iterator
			for(String bind : binds) result+=","+bind;
			result=result.substring(1);
			return "SELECT "+result;
		}
		if(method.equals("filter()")){
			LambdaValue include = (LambdaValue) value.getParameters().get(0);
			Value condition = (Value) include.getOperations().get(0);
			return toStringCondition(condition);
		}
			
		return null;
	}
	
	private String toStringCondition(Value value){
		if(value instanceof BiValue){
			BiValue bool = (BiValue)value;
			String action = bool.getAction();
			if(action.equals("||")) return toStringCondition(bool.left())+" OR "+toStringCondition(bool.right());
			if(action.equals("&&")) return toStringCondition(bool.left())+" AND "+toStringCondition(bool.right());
			if(action.equals("==")) return toStringCondition(bool.left())+" = "+toStringCondition(bool.right());
			throw new RuntimeException("unsuported operation "+action);
		}
		if(value instanceof MethodCallValue){ //take method call instead of value
			MethodCallValue call = (MethodCallValue)value;
			String action = call.getToken().getValue();
			if(action.equals("contains()")) return call.base().getToken().getValue()+" LIKE \"%?%\"";
			if(action.equals("startsWith()")) return call.base().getToken().getValue()+" LIKE \"?%\"";
			if(action.equals("endsWith()")) return call.base().getToken().getValue()+" LIKE \"%?\"";
			return action;
		}
		return "default "+value.getClass();
	}
	
	public String build(){
		this.select = toStringOperation(calls.get(0));
		this.where = toStringOperation(calls.get(1));
		return select+" WHERE "+where;
	}

}
