package dev.peerat.mapping.providers.mysql;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import dev.peerat.mapping.providers.Provider;
import dev.peerat.mapping.providers.ProviderManager;
import dev.peerat.parser.ElementBuilder.Builder;
import dev.peerat.parser.Token;
import dev.peerat.parser.java.BuildedJavaFile;
import dev.peerat.parser.java.Class;
import dev.peerat.parser.java.Function;
import dev.peerat.parser.java.Value;
import dev.peerat.parser.java.Value.MethodCallValue;
import dev.peerat.parser.java.operation.ReturnOperation;

public class MySQLProvider implements Provider{
	
	static{
		ProviderManager.getInstance().register("mysql", new MySQLProvider());
	}
	
	private List<String> treasures;
	private File repo;
	private List<MySQLRequestBuilder> requests;
	
	public MySQLProvider(){
		this.treasures = new ArrayList<>();
		this.requests = new ArrayList<>();
	}
	
	@Override
	public void init(File file) throws Exception{
		repo = new File(file, "/dev/peerat/mapping/mysql/MySQlRepository.java");
		repo.getParentFile().mkdirs();
		repo.createNewFile();
	}
	

	@Override
	public void mapTreasure(Class clazz){
		treasures.add(clazz.getName().getValue());
	}

	@Override
	public void mapBottle(Class clazz, Function function){
		ReturnOperation operation = (ReturnOperation) function.getElements().get(0);
		Value value = operation.getValue();
		if(value instanceof MethodCallValue){
			MethodCallValue call = (MethodCallValue)value;
			List<MethodCallValue> list = new LinkedList<>();
			while(call.base() instanceof MethodCallValue){
				list.add(0, call);
				call = (MethodCallValue) call.base();
			}
			list.add(0, call);
			MySQLRequestBuilder builder = new MySQLRequestBuilder(function, list);
			this.requests.add(builder);
		}
	}

	@Override
	public void mapUses(Class clazz, Function function){
		
	}


	@Override
	public void complete(){
		try{
			BuildedJavaFile file = new BuildedJavaFile();
			file.setPackage("dev.peerat.mapping.sql.MySQlRepository");
//			
//			file.addImport("");
//			
			Class clazz = new Class(null, new Token(0,0,"MySQlRepository",null));
			
			for(MySQLRequestBuilder request : this.requests){
				Function base = request.getFunction();
				Function func = new Function(
						null,
						Modifier.PUBLIC,
						null,
						new Token(0,0,base.getReturnType().getValue(), null),
						new Token(0,0,base.getName().getValue(),null),
						base.getParameters());
				clazz.addFunction(func);
			}
			
			file.setClass(clazz);
			
			Builder builder = new Builder();
			file.build().build(builder);
			BufferedWriter writer = new BufferedWriter(new FileWriter(this.repo));
			builder.build(writer);
			writer.flush();
			writer.close();
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
	}
}
